/*******************************************************
 *  Zeit Shotai Kurabu interface routine for xdvi pl.17
 *  Written by Y. Hayashi   (hayashi@me.aoyama.ac.jp)
 ******************************************************/

#include "xdvi-config.h"
#include "xdvi.h"
#include "dvi-init.h"
#include "util.h"

#ifndef       X_NOT_STDC_ENV
#include <stdlib.h>
#endif

#include "ptexmap.h"
#include "zeit.h"
#include "jfm.h"

#ifdef PTEX
#include "ft2.h"

#ifdef	DEBUGZEIT
void dispfont(struct bitmap *bm)
{
    int w, h;
    unsigned char mask;
    unsigned char *p;

    for (h = 0; h < bm->h; ++h) {
	p = (unsigned char *) &bm->bits[bm->bytes_wide * h];
	for (w = 0; w < bm->w; p++) {
	    for (mask = 0x80; w < bm->w && mask != 0; w++, mask >>= 1)
		putc(*p & mask ? '*' : '.', stderr);
	    putc('|', stderr);
	}
	putc('\n', stderr);
    }
}
#endif	/* DEBUGZEIT */

static void read_ZEIT_char(struct font *fontp, wide_ubyte ch)
{
    struct glyph *g;
    int ch2;
    int ascend;

#ifdef	DEBUGZEIT
    if (resource.list_fonts)
	fprintf(stderr, "zeit raster: %x in %s\n", ch, fontp->fontname);
#endif  /* DEBUGZEIT */
    ch2 = jisindex(ch);

    g = fontp->kglyph[ch2];
    if (g == NULL) {
	struct glyph *orig = fontp->kglyph[0];
	ASSERT(orig != NULL, "Unexpected error.");
	g = fontp->kglyph[ch2] = xcalloc(1, sizeof(struct glyph));
	g->bitmap.w = orig->bitmap.w;
	g->bitmap.h = orig->bitmap.h;
	g->x        = orig->x;
	g->y        = orig->y;
	g->dvi_adv  = orig->dvi_adv;
    }
    alloc_bitmap(&g->bitmap);
    ascend = ft2_get_bitmap(ch, fontp->ft2_index, &g->bitmap);
#ifdef	DEBUGZEIT
    dispfont(bm);
#endif  /* DEBUGZEIT */
#ifndef WORDS_BIGENDIAN
    order_reverse_bitmap(&g->bitmap);
#endif  /* !WORDS_BIGENDIAN */
    g->tdir = fontp->dir;
    if (fontp->dir) {  /* tate */
	int swap = g->x;
	g->x     = g->y;
	g->y     = swap;
    } else {  /* yoko */
	g->y = ascend;
    }
}

int read_ZEIT_index(struct font *fontp)
{
    int n;
    struct glyph *g;
    struct jfm *jfm;
    const double dimconv = fontp->dimconv;

    fontp->read_char = read_ZEIT_char;
    fontp->kglyph = xcalloc(KTABLESIZE, sizeof(struct glyph *));

    jfm = read_jfm(fontp->file, fontp->fontname);
    if (jfm == NULL) return -1;
    fontp->ft2_index = ft2_open_font(getkanjifont(fontp->fontname));
    if (fontp->ft2_index < 0) return -1;

    for (n=0; n<jfm->table[J_NT]; n++) {
	int code  = jisindex(jfm->type[n].code);
	int index =  jfm->type[n].index;
	int width =  jfm->width[jfm->info[index].width_ix];
	int height= jfm->height[jfm->info[index].height_depth_ix / 0x10];
	int depth =  jfm->depth[jfm->info[index].height_depth_ix % 0x10];

	g = xcalloc(1, sizeof(struct glyph));
	g->bitmap.w = (int)(dimconv * width) >> 16;
	g->bitmap.h = (int)(dimconv * (height + depth)) >> 16;
	if (g->bitmap.w != g->bitmap.h) {
	    fprintf(stderr, "Warning: '%s' (code=0x%X) is not square.\n",
		    fontp->fontname, code);
	}
	g->bitmap.bits = NULL;
	g->x = 0;
	g->y = (int) (dimconv * height) >> 16;
	g->dvi_adv = dimconv * width;
	fontp->kglyph[code] = g;
    }
    g = fontp->kglyph[0];
    alloc_bitmap(&g->bitmap);
    fill_bitmap(&g->bitmap);
    return 0;
}
#endif  /* PTEX */
